"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
require("mocha");
const chai = tslib_1.__importStar(require("chai"));
const shared_1 = require("../../shared");
const core_utils_1 = require("../../../../shared/core-utils");
const server_commands_1 = require("../../../../shared/server-commands");
const expect = chai.expect;
function createLive(server, permanent) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const attributes = {
            channelId: server.store.channel.id,
            privacy: 1,
            name: 'my super live',
            saveReplay: true,
            permanentLive: permanent
        };
        const { uuid } = yield server.live.create({ fields: attributes });
        return uuid;
    });
}
function checkFiles(files) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        for (const file of files) {
            (0, shared_1.expectStartWith)(file.fileUrl, server_commands_1.ObjectStorageCommand.getPlaylistBaseUrl());
            yield (0, server_commands_1.makeRawRequest)(file.fileUrl, 200);
        }
    });
}
function getFiles(server, videoUUID) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const video = yield server.videos.get({ id: videoUUID });
        expect(video.files).to.have.lengthOf(0);
        expect(video.streamingPlaylists).to.have.lengthOf(1);
        return video.streamingPlaylists[0].files;
    });
}
function streamAndEnd(servers, liveUUID) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const ffmpegCommand = yield servers[0].live.sendRTMPStreamInVideo({ videoId: liveUUID });
        yield (0, server_commands_1.waitUntilLivePublishedOnAllServers)(servers, liveUUID);
        const videoLiveDetails = yield servers[0].videos.get({ id: liveUUID });
        const liveDetails = yield servers[0].live.get({ videoId: liveUUID });
        yield (0, server_commands_1.stopFfmpeg)(ffmpegCommand);
        if (liveDetails.permanentLive) {
            yield (0, server_commands_1.waitUntilLiveWaitingOnAllServers)(servers, liveUUID);
        }
        else {
            yield (0, server_commands_1.waitUntilLiveReplacedByReplayOnAllServers)(servers, liveUUID);
        }
        yield (0, server_commands_1.waitJobs)(servers);
        return { videoLiveDetails, liveDetails };
    });
}
describe('Object storage for lives', function () {
    if ((0, core_utils_1.areObjectStorageTestsDisabled)())
        return;
    let servers;
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(120000);
            yield server_commands_1.ObjectStorageCommand.prepareDefaultBuckets();
            servers = yield (0, server_commands_1.createMultipleServers)(2, server_commands_1.ObjectStorageCommand.getDefaultConfig());
            yield (0, server_commands_1.setAccessTokensToServers)(servers);
            yield (0, server_commands_1.setDefaultVideoChannel)(servers);
            yield (0, server_commands_1.doubleFollow)(servers[0], servers[1]);
            yield servers[0].config.enableTranscoding();
        });
    });
    describe('Without live transcoding', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let videoUUID;
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield servers[0].config.enableLive({ transcoding: false });
                    videoUUID = yield createLive(servers[0], false);
                });
            });
            it('Should create a live and save the replay on object storage', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(220000);
                    yield streamAndEnd(servers, videoUUID);
                    for (const server of servers) {
                        const files = yield getFiles(server, videoUUID);
                        expect(files).to.have.lengthOf(1);
                        yield checkFiles(files);
                    }
                });
            });
        });
    });
    describe('With live transcoding', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let videoUUIDPermanent;
            let videoUUIDNonPermanent;
            before(function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    yield servers[0].config.enableLive({ transcoding: true });
                    videoUUIDPermanent = yield createLive(servers[0], true);
                    videoUUIDNonPermanent = yield createLive(servers[0], false);
                });
            });
            it('Should create a live and save the replay on object storage', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(240000);
                    yield streamAndEnd(servers, videoUUIDNonPermanent);
                    for (const server of servers) {
                        const files = yield getFiles(server, videoUUIDNonPermanent);
                        expect(files).to.have.lengthOf(5);
                        yield checkFiles(files);
                    }
                });
            });
            it('Should create a live and save the replay of permanent live on object storage', function () {
                return tslib_1.__awaiter(this, void 0, void 0, function* () {
                    this.timeout(240000);
                    const { videoLiveDetails } = yield streamAndEnd(servers, videoUUIDPermanent);
                    const replay = yield (0, server_commands_1.findExternalSavedVideo)(servers[0], videoLiveDetails);
                    for (const server of servers) {
                        const files = yield getFiles(server, replay.uuid);
                        expect(files).to.have.lengthOf(5);
                        yield checkFiles(files);
                    }
                });
            });
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.killallServers)(servers);
        });
    });
});
//# sourceMappingURL=live.js.map